From d67fb15fdcd7a742ce0bb7963ddcfea430f7ef19 Mon Sep 17 00:00:00 2001
From: Noam Postavsky <npostavs@gmail.com>
Date: Mon, 20 Jun 2016 20:08:15 -0400
Subject: [PATCH] Set Vinitial_environment before changing env vars

* src/callproc.c (make_list_from_environ): Renamed from
set_initial_environment, just return a Lisp list of the environment
instead of setting of Vprocess_environment and Vinitial_environment
directly.
* src/emacs.c (main): Set Vinitial_environment before init_environment
is called so that it gets the initial environment inherited from the
parent process and remains unaffected by modifications Emacs
performs (Bug #10980).
---
 src/callproc.c | 12 +++++-------
 src/emacs.c    | 14 ++++++++------
 src/lisp.h     |  2 +-
 3 files changed, 14 insertions(+), 14 deletions(-)

diff --git a/src/callproc.c b/src/callproc.c
index 0729782..924b755 100644
--- a/src/callproc.c
+++ b/src/callproc.c
@@ -1577,16 +1577,14 @@ init_callproc (void)
 #endif
 }
 
-void
-set_initial_environment (void)
+Lisp_Object
+make_list_from_environ (void)
 {
   char **envp;
+  Lisp_Object list = Qnil;
   for (envp = environ; *envp; envp++)
-    Vprocess_environment = Fcons (build_string (*envp),
-				  Vprocess_environment);
-  /* Ideally, the `copy' shouldn't be necessary, but it seems it's frequent
-     to use `delete' and friends on process-environment.  */
-  Vinitial_environment = Fcopy_sequence (Vprocess_environment);
+    list = Fcons (build_string (*envp), list);
+  return list;
 }
 
 void
diff --git a/src/emacs.c b/src/emacs.c
index bb85733..1dbb3f6 100644
--- a/src/emacs.c
+++ b/src/emacs.c
@@ -1210,10 +1210,17 @@ Using an Emacs configured with --with-x-toolkit=lucid does not have this problem
 #ifdef HAVE_WINDOW_SYSTEM
       init_fringe_once ();	/* Swap bitmaps if necessary.  */
 #endif /* HAVE_WINDOW_SYSTEM */
+
+      /* Initialize and GC-protect Vinitial_environment and Vprocess_environment
+         before filling them in.  */
+      syms_of_callproc ();
     }
 
   init_alloc ();
 
+  if (! dumping)
+    Vinitial_environment = make_list_from_environ ();
+
   if (do_initial_setlocale)
     {
       fixup_locale ();
@@ -1366,16 +1373,11 @@ Using an Emacs configured with --with-x-toolkit=lucid does not have this problem
   ns_init_locale ();
 #endif
 
-  /* Initialize and GC-protect Vinitial_environment and
-     Vprocess_environment before set_initial_environment fills them
-     in.  */
-  if (!initialized)
-    syms_of_callproc ();
   /* egetenv is a pretty low-level facility, which may get called in
      many circumstances; it seems flimsy to put off initializing it
      until calling init_callproc.  Do not do it when dumping.  */
   if (! dumping)
-    set_initial_environment ();
+    Vprocess_environment = make_list_from_environ ();
 
   /* AIX crashes are reported in system versions 3.2.3 and 3.2.4
      if this is not done.  Do it after set_global_environment so that we
diff --git a/src/lisp.h b/src/lisp.h
index e0eb52a..33e1f70 100644
--- a/src/lisp.h
+++ b/src/lisp.h
@@ -4199,7 +4199,7 @@ extern void setup_process_coding_systems (Lisp_Object);
 extern int child_setup (int, int, int, char **, bool, Lisp_Object);
 extern void init_callproc_1 (void);
 extern void init_callproc (void);
-extern void set_initial_environment (void);
+extern Lisp_Object make_list_from_environ (void);
 extern void syms_of_callproc (void);
 
 /* Defined in doc.c.  */
-- 
2.6.2.windows.1

